#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_workflow.interfaces import IWorkflowManagedContent, IWorkflow, IWorkflowTransitionInfo, IWorkflowCommentInfo, \
    IWorkflowInfo, IWorkflowState
from z3c.form.interfaces import HIDDEN_MODE

# import packages
from pyams_form.form import AJAXAddForm
from pyams_utils.registry import query_utility
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_zmi.form import AdminDialogAddForm
from pyramid.decorator import reify
from zope.lifecycleevent import ObjectModifiedEvent
from z3c.form import field


class WorkflowContentTransitionForm(AdminDialogAddForm):
    """Workflow content transition form"""

    prefix = 'workflow.'

    @reify
    def transition(self):
        parent = get_parent(self.context, IWorkflowManagedContent)
        workflow = query_utility(IWorkflow, name=parent.workflow_name)
        return workflow.get_transition_by_id(self.request.params.get('workflow.widgets.transition_id'))

    @property
    def legend(self):
        return self.request.localizer.translate(self.transition.title)

    fields = field.Fields(IWorkflowTransitionInfo) + \
             field.Fields(IWorkflowCommentInfo)

    @property
    def edit_permission(self):
        return self.transition.permission

    @property
    def icon_css_class(self):
        return self.transition.user_data.get('menu_css_class')

    def updateWidgets(self, prefix=None):
        super(WorkflowContentTransitionForm, self).updateWidgets(prefix)
        if 'transition_id' in self.widgets:
            self.widgets['transition_id'].mode = HIDDEN_MODE
            self.widgets['transition_id'].value = self.request.params.get('workflow.widgets.transition_id')
        if 'comment' in self.widgets:
            self.widgets['comment'].widget_css_class = 'textarea'

    def updateActions(self):
        super(WorkflowContentTransitionForm, self).updateActions()
        if 'action' in self.actions:
            self.actions['action'].addClass('btn-primary')

    def createAndAdd(self, data):
        data = data.get(self, data)
        info = IWorkflowInfo(self.context)
        info.fire_transition(self.transition.transition_id, comment=data.get('comment'))
        info.fire_automatic()
        IWorkflowState(self.context).state_urgency = data.get('urgent_request') or False
        self.request.registry.notify(ObjectModifiedEvent(self.context))
        return info

    def nextURL(self):
        return absolute_url(self.context, self.request, 'summary.html')


class WorkflowContentTransitionAJAXForm(AJAXAddForm):
    """Workflow content transition AJAX form"""

    def get_ajax_output(self, changes):
        return {'status': 'reload'}
